// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================



module audio (
	CLOCK_I,
	VOL_CLOCK_I,
	DATA_O,
	END_I,
	RESETN_I,
	GO_O,
	ACK_I,
	FREQUENCY_I,
	LEFT_VOLUME_UP_I,
	LEFT_VOLUME_DOWN_I,
	RIGHT_VOLUME_UP_I,
	RIGHT_VOLUME_DOWN_I,	
	CODEC_READY_O,
	VOLUME_O
);
	input  CLOCK_I;
	input  VOL_CLOCK_I;	
	input  END_I;
	input  RESETN_I;
	input [1:0] FREQUENCY_I;
	input LEFT_VOLUME_UP_I;
	input LEFT_VOLUME_DOWN_I;
	input RIGHT_VOLUME_UP_I;
	input RIGHT_VOLUME_DOWN_I;
	input ACK_I;
	
	output [23:0] DATA_O;
	reg [23:0] DATA_O;
	output GO_O;
	reg GO_O;
	output CODEC_READY_O;
	reg CODEC_READY_O;
	
	output [7:0] VOLUME_O;

//	LUT Data Number
parameter	LUT_SIZE	=	11;
//	Audio Data Index
parameter	RESET_CODEC	=	0;
parameter	POWER_ON	=	1;
parameter	SET_FORMAT	=	2;
parameter	SAMPLE_CTRL	=	3;
parameter	A_PATH_CTRL	=	4;
parameter	D_PATH_CTRL	=	5;
parameter	SET_LIN_L	=	6;
parameter	SET_LIN_R	=	7;
parameter	SET_HEAD_L	=	8;
parameter	SET_HEAD_R	=	9;
parameter	SET_ACTIVE	=	10;

reg	[15:0]	LUT_DATA;
reg	[3:0]	LUT_INDEX;
reg	[3:0]	mSetup_ST;

reg [7:0] left_vol, right_vol;
reg [1:0] cycle_counter;

assign VOLUME_O = {left_vol[6:3], right_vol[6:3]};

always @(posedge VOL_CLOCK_I or negedge RESETN_I) begin
	if (RESETN_I == 1'b0) begin
		left_vol <= 8'h70;
		right_vol <= 8'h70;
	end else begin
		if (LEFT_VOLUME_UP_I == 1'b1 && left_vol < 8'h78) begin
			left_vol <= left_vol + 8'd7;
		end
		if (LEFT_VOLUME_DOWN_I == 1'b1 && left_vol >= 8'h27) begin
			left_vol <= left_vol - 8'd7;		
		end
		if (RIGHT_VOLUME_UP_I == 1'b1 && right_vol < 8'h78) begin
			right_vol <= right_vol + 8'd7;
		end
		if (RIGHT_VOLUME_DOWN_I == 1'b1 && right_vol >= 8'h27) begin
			right_vol <= right_vol - 8'd7;		
		end

	end
end

//////////////////////	Config Control	////////////////////////////
always@(posedge CLOCK_I or negedge RESETN_I)
begin
	if(!RESETN_I)
	begin
		LUT_INDEX	<=	4'd0;
		mSetup_ST	<=	2'd0;
		GO_O		<=	1'b0;
		CODEC_READY_O <= 1'b0;
		cycle_counter <= 2'd0;
	end
	else
	begin
		if(LUT_INDEX >= LUT_SIZE && CODEC_READY_O == 1'b0) CODEC_READY_O <= 1'b1;
		
		if(CODEC_READY_O == 1'b0)		
		begin
			case(mSetup_ST)
			2'd0:	begin
					DATA_O	<=	{8'h34,LUT_DATA};
					GO_O		<=	1'b1;
					mSetup_ST	<=	2'd1;
				end
			2'd1:	begin
					if(END_I)
					begin
						if(!ACK_I)
						mSetup_ST	<=	2'd2;
						else
						mSetup_ST	<=	2'd0;							
						GO_O		<=	1'b0;
					end
				end
			2'd2:	begin
					LUT_INDEX	<=	LUT_INDEX+ 4'd1;
					mSetup_ST	<=	2'd0;
				end
			endcase
		end else begin
			// Finish setting up
	
			case(mSetup_ST)
			2'd0:	begin
					case (cycle_counter)
					2'd0: DATA_O <= {8'h34, 8'h04, 1'b0, left_vol[6:0]}; // left headphone volume
					2'd1: DATA_O <= {8'h34, 8'h06, 1'b0, right_vol[6:0]}; // right headphone volume
					default: begin // sampling control
						DATA_O[23:8] <= {8'h34, 8'h10};
						case (FREQUENCY_I)
/* USB_MODE 
						2'd0: DATA_O[7:0] <= 8'h23;//44 K
						2'd1: DATA_O[7:0] <= 8'h01;//48 K
						2'd2: DATA_O[7:0] <= 8'h19;//32 K
						default: DATA_O[7:0] <= 8'h0D;//8 K
*/						
/* Normal_MODE */
						2'd0: DATA_O[7:0] <= 8'h22;//44 K
						2'd1: DATA_O[7:0] <= 8'h02;//48 K
						2'd2: DATA_O[7:0] <= 8'h1A;//32 K
						default: DATA_O[7:0] <= 8'h0E;//8 K

						endcase
					end
					endcase
					GO_O		<=	1'b1;
					mSetup_ST	<=	2'b01;
				end
			2'd1:	begin
					if(END_I)
					begin
						if(!ACK_I)
						mSetup_ST	<=	2'd2;
						else
						mSetup_ST	<=	2'd0;							
						GO_O		<=	1'b0;
					end
				end
			2'd2:	begin
					cycle_counter <= cycle_counter + 2'd1;
					mSetup_ST	<=	2'd0;
				end
			endcase
			
		end
	end
end

/////////////////////	Config Data LUT	  //////////////////////////	
always
begin
	case(LUT_INDEX)
	//	Audio Config Data
	RESET_CODEC	:	LUT_DATA	<=	16'h1E00; // Reset the device to known state
	POWER_ON	:	LUT_DATA	<=	16'h0C07; // Power on the output side
	SET_FORMAT	:	LUT_DATA	<=	16'h0E41; // Master mode, MSB-First, left justified, 16 bits
	A_PATH_CTRL	:	LUT_DATA	<=	16'h0810; // only select DAC
	D_PATH_CTRL	:	LUT_DATA	<=	16'h0A00; // disable by default on DAC
//	SAMPLE_CTRL	:	LUT_DATA	<=	16'h1001; // USB_mode 48 KHz
	SAMPLE_CTRL	:	LUT_DATA	<=	16'h1002; // Normal_mode 48 KHz	
	SET_LIN_L	:	LUT_DATA	<=	16'h0000; // mute line in left
	SET_LIN_R	:	LUT_DATA	<=	16'h0200; // mute line in right
	SET_HEAD_L	:	LUT_DATA	<=	16'h0470; // unmute headphone left
	SET_HEAD_R	:	LUT_DATA	<=	16'h0670; // unmute headphone right
	default		:	LUT_DATA	<=	16'h1201; // Activate codec
	endcase
end

endmodule

